package main

import (
	"bufio"
	"context"
	"io"
	"net/http"
	"os/exec"
	"strings"

	"github.com/google/uuid"
	"github.com/sirupsen/logrus"
	"github.com/spf13/cobra"
)



func main() {
	logrus.SetLevel(logrus.DebugLevel)
	cmd := cobra.Command{
		Use: "webcmd [command]",
		Short: "WebCmd is a web server exposing a endpoint running the given command.",
		Args: cobra.MinimumNArgs(1),
		Run: func(cmd *cobra.Command, args []string) {
			ctx, cancel := context.WithCancel(context.Background())
			defer cancel()
			address, err := cmd.Flags().GetString("address")
			if err != nil {
				logrus.Fatal(err)
			}
			command := strings.Join(args[0:], " ")
			log := logrus.WithField("command", command)
			http.HandleFunc("/", func(w http.ResponseWriter, r *http.Request) {
				log := log.WithField("remoteAddress", r.RemoteAddr).WithField("id", uuid.New().String())
				log.Infof("New Incoming request")
				c := exec.CommandContext(ctx, args[0], args[1:]...)
				outp, err := c.StdoutPipe()
				if err != nil {
					log.Error(err)
					http.Error(w, err.Error(), 500)
					return
				}
				errp, err := c.StderrPipe()
				if err != nil {
					log.Error(err)
					http.Error(w, err.Error(), 500)
					return
				}
				if err := c.Start(); err != nil {
					log.Error(err)
					http.Error(w, err.Error(), 500)
					return
				}
				go func() {
					go logOut(log, outp)
					go logOut(log, errp)
					if err := c.Wait(); err != nil {
						log.Error(err)
					}
				}()
			})
			logrus.Infof("Starting server on %s for command '%s'", address, command)
			if err := http.ListenAndServe(address, nil); err != nil {
				logrus.Fatal(err)
			}
		},
	}
	cmd.Flags().StringP("address", "a", ":8080", "the address the server will listen on")
	cmd.Execute()
}

func logOut(log logrus.FieldLogger, out io.ReadCloser) {
	outr := bufio.NewReader(out)
	for {
		l, _, err := outr.ReadLine()
		if err != nil {
			return
		}
		log.Debug(string(l))
	}
}
